
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '0.0.3'
version = '0.0.3'
full_version = '0.0.3.dev20160127183730'
git_revision = '81e54b3d0864ee539966d42a16bc8b45b3e1e319'
release = False

if not release:
    version = full_version
